div#view{flex:1;background-color:#000;border:1px solid #000;color:#fff;height:100vh;width:100vw}.icon{width:24px;height:24px;color:#d8d8d8}#navigation-menu{z-index:1000;position:fixed;top:5vh;right:1vw;width:25vw;max-height:90vh;overflow-y:auto}
